@@ -43,7 +43,7 @@ def make_list(thing):
 final_date_cache = {}
 
 
-def get_final_date(inst, create_date, duration, holiday_calendar=[]):
+def get_final_date(inst, create_date, duration, holiday_calendar=None):
     global final_date_cache
     cache_key = (inst, create_date, duration, holiday_calendar)
     if cache_key in final_date_cache:
