@@ -44,6 +44,7 @@ from gs_quant.markets.indices_utils import BasketType, IndicesDatasets
 from gs_quant.markets.position_set import PositionSet, Position
 from gs_quant.markets.report import PerformanceReport, FactorRiskReport, Report, ThematicReport, \
     flatten_results_into_df, get_thematic_breakdown_as_df
+from gs_quant.models.risk_model import MacroRiskModel, ReturnFormat
 from gs_quant.session import GsSession
 from gs_quant.target.data import DataQuery
 from gs_quant.target.reports import ReportStatus, ReportType
@@ -151,7 +152,7 @@ class Entity(metaclass=ABCMeta):
         return self.__entity
 
     def get_unique_entity_key(self) -> EntityKey:
-        return EntityKey(self.__id, self.__entity_type)
+        return EntityKey(self.get_marquee_id(), self.__entity_type)
 
     def get_data_coordinate(self,
                             measure: Union[DataMeasure, str],
@@ -367,8 +368,11 @@ class PositionedEntity(metaclass=ABCMeta):
                                   date: dt.date,
                                   position_type: PositionType = PositionType.CLOSE) -> PositionSet:
         if self.positioned_entity_type == EntityType.ASSET:
-            response = GsAssetApi.get_asset_positions_for_date(self.id, date, position_type)[0]
-            return PositionSet.from_target(response)
+            response = GsAssetApi.get_asset_positions_for_date(self.id, date, position_type)
+            if(len(response) == 0):
+                _logger.info("No positions available for {}".format(date))
+                return PositionSet([], date=date)
+            return PositionSet.from_target(response[0])
         if self.positioned_entity_type == EntityType.PORTFOLIO:
             response = GsPortfolioApi.get_positions_for_date(portfolio_id=self.id,
                                                              position_date=date,
@@ -382,6 +386,9 @@ class PositionedEntity(metaclass=ABCMeta):
                           position_type: PositionType = PositionType.CLOSE) -> List[PositionSet]:
         if self.positioned_entity_type == EntityType.ASSET:
             response = GsAssetApi.get_asset_positions_for_dates(self.id, start, end, position_type)
+            if(len(response) == 0):
+                _logger.info("No positions available in the date range {} - {}".format(start, end))
+                return []
             return [PositionSet.from_target(position_set) for position_set in response]
         if self.positioned_entity_type == EntityType.PORTFOLIO:
             response = GsPortfolioApi.get_positions(portfolio_id=self.id,
@@ -407,7 +414,8 @@ class PositionedEntity(metaclass=ABCMeta):
                     new_sets.append(pos_set)
                 else:
                     new_sets.append(self._convert_pos_set_with_weights(pos_set, currency))
-            GsPortfolioApi.update_positions(portfolio_id=self.id, position_sets=[p.to_target() for p in new_sets])
+            GsPortfolioApi.update_positions(portfolio_id=self.id, position_sets=[p.to_target() for p in new_sets],
+                                            net_positions=net_positions)
             time.sleep(3)
         else:
             raise NotImplementedError
@@ -451,11 +459,15 @@ class PositionedEntity(metaclass=ABCMeta):
                            start: dt.date = DateLimit.LOW_LIMIT.value,
                            end: dt.date = dt.date.today(),
                            fields: [str] = None,
-                           position_type: PositionType = PositionType.CLOSE) -> List[Dict]:
+                           position_type: PositionType = PositionType.CLOSE,
+                           include_all_business_days: bool = False) -> List[Dict]:
         if self.positioned_entity_type == EntityType.ASSET:
+            if include_all_business_days:
+                raise MqError('"include_all_business_days" cannot be set to true for assets')
             return GsIndexApi.get_positions_data(self.id, start, end, fields, position_type)
         if self.positioned_entity_type == EntityType.PORTFOLIO:
-            return GsPortfolioApi.get_positions_data(self.id, start, end, fields, position_type)
+            return GsPortfolioApi.get_positions_data(self.id, start, end, fields,
+                                                     position_type, include_all_business_days)
         raise NotImplementedError
 
     def get_position_dates(self) -> Tuple[dt.date, ...]:
@@ -511,14 +523,20 @@ class PositionedEntity(metaclass=ABCMeta):
 
     def get_factor_risk_report(self,
                                risk_model_id: str = None,
-                               fx_hedged: bool = None) -> FactorRiskReport:
+                               fx_hedged: bool = None,
+                               benchmark_id: str = None) -> FactorRiskReport:
         position_source_type = self.positioned_entity_type.value.capitalize()
         reports = self.get_factor_risk_reports(fx_hedged=fx_hedged)
         if risk_model_id:
             reports = [report for report in reports if report.parameters.risk_model == risk_model_id]
+        reports = [report for report in reports if report.parameters.benchmark == benchmark_id]
+        if len(reports) == 0:
+            raise MqError(f'This {position_source_type} has no factor risk reports that match '
+                          'your parameters. Please edit the risk model ID, fxHedged, and/or benchmark value in the '
+                          'function parameters.')
         if len(reports) > 1:
             raise MqError(f'This {position_source_type} has more than one factor risk report that matches '
-                          'your parameters. Please specify the risk model ID and fxHedged value in the '
+                          'your parameters. Please specify the risk model ID, fxHedged, and/or benchmark value in the '
                           'function parameters.')
         return reports[0]
 
@@ -840,8 +858,6 @@ class PositionedEntity(metaclass=ABCMeta):
                                    end_date: dt.date = None,
                                    basket_ids: List[str] = None,
                                    regions: List[Region] = None) -> pd.DataFrame:
-        if self.positioned_entity_type == EntityType.PORTFOLIO:
-            raise NotImplementedError
         results = GsThematicApi.get_thematics(entity_id=self.id,
                                               start_date=start_date,
                                               end_date=end_date,
@@ -855,8 +871,6 @@ class PositionedEntity(metaclass=ABCMeta):
                                         end_date: dt.date = None,
                                         basket_ids: List[str] = None,
                                         regions: List[Region] = None) -> pd.DataFrame:
-        if self.positioned_entity_type == EntityType.PORTFOLIO:
-            raise NotImplementedError
         results = GsThematicApi.get_thematics(entity_id=self.id,
                                               start_date=start_date,
                                               end_date=end_date,
@@ -870,8 +884,6 @@ class PositionedEntity(metaclass=ABCMeta):
                                            end_date: dt.date = None,
                                            basket_ids: List[str] = None,
                                            regions: List[Region] = None) -> pd.DataFrame:
-        if self.positioned_entity_type == EntityType.PORTFOLIO:
-            raise NotImplementedError
         results = GsThematicApi.get_thematics(entity_id=self.id,
                                               start_date=start_date,
                                               end_date=end_date,
@@ -890,6 +902,13 @@ class PositionedEntity(metaclass=ABCMeta):
         :param basket_id: GS flagship basket's unique Marquee ID
         :return: a Pandas DataFrame with results
         """
-        if self.positioned_entity_type == EntityType.PORTFOLIO:
-            raise NotImplementedError
         return get_thematic_breakdown_as_df(entity_id=self.id, date=date, basket_id=basket_id)
+
+    def get_macro_exposure_table(self,
+                                 macro_risk_model: MacroRiskModel,
+                                 date: dt.date,
+                                 factors: List[str] = [],
+                                 factors_by_name: bool = True,
+                                 group_by_macro_bucket: bool = True,
+                                 return_format: ReturnFormat = ReturnFormat.DATA_FRAME) -> Union[Dict, pd.DataFrame]:
+        raise NotImplementedError
