@@ -25,18 +25,19 @@ from inspect import signature
 from itertools import zip_longest
 from typing import Optional, Union
 
-from tqdm import tqdm
-
 from gs_quant.base import InstrumentBase, RiskKey, Scenario, get_enum_value
 from gs_quant.common import PricingLocation, RiskMeasure
 from gs_quant.context_base import ContextBaseWithDefault
-from gs_quant.datetime.date import business_day_offset
+from gs_quant.datetime.date import business_day_offset, today
 from gs_quant.risk import CompositeScenario, DataFrameWithInfo, ErrorValue, FloatWithInfo, MarketDataScenario, \
     StringWithInfo
-from gs_quant.risk.results import PricingFuture
+from gs_quant.risk.result_handlers import extract_val_from_dataframe_with_info
+from gs_quant.risk.results import PricingFuture, MultipleScenarioResult
 from gs_quant.session import GsSession
-from gs_quant.target.common import PricingDateAndMarketDataAsOf
+from gs_quant.target.common import PricingDateAndMarketDataAsOf, MultiScenario
 from gs_quant.target.risk import RiskPosition, RiskRequest, RiskRequestParameters
+from tqdm import tqdm
+
 from .markets import CloseMarket, LiveMarket, Market, close_market_date, OverlayMarket, RelativeMarket
 
 _logger = logging.getLogger(__name__)
@@ -81,6 +82,7 @@ class PricingContext(ContextBaseWithDefault):
                  is_batch: bool = False,
                  use_cache: bool = False,
                  visible_to_gs: Optional[bool] = None,
+                 request_priority: Optional[int] = None,
                  csa_term: Optional[str] = None,
                  timeout: Optional[int] = None,
                  market: Optional[Market] = None,
@@ -98,6 +100,7 @@ class PricingContext(ContextBaseWithDefault):
             It can be used with is_async=True|False (defaults to False)
         :param use_cache: store results in the pricing cache (defaults to False)
         :param visible_to_gs: are the contents of risk requests visible to GS (defaults to False)
+        :param request_priority: the priority of risk requests
         :param csa_term: the csa under which the calculations are made. Default is local ccy ois index
         :param market_data_location: the location for sourcing market data ('NYC', 'LDN' or 'HKG' (defaults to LDN)
         :param timeout: the timeout for batch operations
@@ -138,7 +141,9 @@ class PricingContext(ContextBaseWithDefault):
                 get_enum_value(PricingLocation, market_data_location):
             raise ValueError('market.location and market_data_location cannot be different')
 
-        if not market and pricing_date and pricing_date > dt.date.today():
+        if not market and pricing_date and pricing_date > dt.date.today() + dt.timedelta(5):
+            # We allow a small tolerance to rolling over weekends/holidays
+            # We should use a calendar but not everyone has access
             raise ValueError(
                 'The PricingContext does not support a pricing_date in the future. Please use the RollFwd Scenario '
                 'to roll the pricing_date to a future date')
@@ -169,8 +174,11 @@ class PricingContext(ContextBaseWithDefault):
             else:
                 market_data_location = market.location
 
-        self.__pricing_date = pricing_date or (self.prior_context.pricing_date if self.prior_context else
-                                               business_day_offset(dt.date.today(), 0, roll='preceding'))
+        market_data_location = get_enum_value(PricingLocation, market_data_location)
+
+        self.__pricing_date = pricing_date or (
+            self.prior_context.pricing_date if self.prior_context else business_day_offset(
+                today(market_data_location), 0, roll='preceding'))
         self.__csa_term = csa_term
         self.__market_behaviour = market_behaviour
         self.__is_async = is_async
@@ -178,14 +186,17 @@ class PricingContext(ContextBaseWithDefault):
         self.__timeout = timeout
         self.__use_cache = use_cache
         self.__visible_to_gs = visible_to_gs
-        self.__market_data_location = get_enum_value(PricingLocation, market_data_location)
+        self.__request_priority = request_priority
+        self.__market_data_location = market_data_location
         self.__market = market or CloseMarket(
             date=close_market_date(self.__market_data_location, self.__pricing_date) if pricing_date else None,
             location=self.__market_data_location if market_data_location else None)
         self.__pending = {}
         self.__show_progress = show_progress
+        self._max_per_batch = 1000
         self._max_concurrent = 1000
         self.__use_server_cache = use_server_cache
+        self._group_by_date = True
 
     def _on_exit(self, exc_type, exc_val, exc_tb):
         if exc_val:
@@ -252,10 +263,10 @@ class PricingContext(ContextBaseWithDefault):
 
                 for (params, scenario, dates_markets, risk_measures), instruments in grouped_requests.items():
                     for insts_chunk in [tuple(filter(None, i)) for i in
-                                        zip_longest(*[iter(instruments)] * self._max_concurrent)]:
+                                        zip_longest(*[iter(instruments)] * self._max_per_batch)]:
                         for dates_chunk in [tuple(filter(None, i)) for i in
                                             zip_longest(*[iter(dates_markets)] * (
-                                                    1 if self._max_concurrent == 1000 else self._max_concurrent))]:
+                                                    1 if self._group_by_date else self._max_per_batch))]:
                             requests.append(RiskRequest(
                                 tuple(RiskPosition(instrument=i, quantity=i.instrument_quantity,
                                                    instrument_name=i.name) for i in insts_chunk),
@@ -267,7 +278,8 @@ class PricingContext(ContextBaseWithDefault):
                                     PricingDateAndMarketDataAsOf(pricing_date=d, market=m)
                                     for d, m in dates_chunk),
                                 request_visible_to_gs=request_visible_to_gs,
-                                use_cache=self.__use_server_cache
+                                use_cache=self.__use_server_cache,
+                                priority=self.__request_priority
                             ))
 
                 requests_for_provider[provider] = requests
@@ -375,10 +387,22 @@ class PricingContext(ContextBaseWithDefault):
 
         future = pending.get((risk_key, instrument))
 
+        multi_scenario = [i for i in Scenario.path if isinstance(i, MultiScenario)]
+
+        def handle_multi_scen_future(result):
+            if isinstance(result.result(), DataFrameWithInfo):
+                # so that we convert the float64 values from dataframe to Floatwithinfos
+                values = extract_val_from_dataframe_with_info(result.result())
+                # replace simplevaltable with futures
+                result.set_result(
+                    MultipleScenarioResult(instrument, {multi_scenario[0].scenarios[v_idx]: v
+                                                        for v_idx, v in enumerate(values)}))
+
         if future is None:
             future = PricingFuture()
             cached_result = PricingCache.get(risk_key, instrument) if self.use_cache else None
-
+            if len(multi_scenario):
+                future.add_done_callback(handle_multi_scen_future)
             if cached_result is not None:
                 future.set_result(cached_result)
             else:
