@@ -17,22 +17,80 @@ import copy
 import datetime as dt
 import logging
 import operator as op
+import weakref
 from concurrent.futures import Future
 from itertools import chain
 from typing import Any, Iterable, Mapping, Optional, Tuple, Union
 
 import pandas as pd
-from gs_quant.base import Priceable, RiskKey, Sentinel, InstrumentBase, is_instance_or_iterable, is_iterable
+from gs_quant.base import Priceable, RiskKey, Sentinel, InstrumentBase, is_instance_or_iterable, is_iterable, Scenario
 from gs_quant.common import RiskMeasure
 from gs_quant.config import DisplayOptions
 from gs_quant.risk import DataFrameWithInfo, ErrorValue, FloatWithInfo, SeriesWithInfo, ResultInfo, \
     ScalarWithInfo, aggregate_results
-from gs_quant.target.common import AssetType
-from more_itertools import unique_everseen
 
 _logger = logging.getLogger(__name__)
 
 
+def get_default_pivots(cls: str, has_dates: bool, multi_measures: bool,
+                       multi_scen: bool, simple_port: bool = None, ori_cols=None):
+    if cls == 'MultipleScenarioResult':
+        return 'value', 'scenario', 'dates' if has_dates else None
+    elif cls == 'MultipleRiskMeasureResult':
+        return 'value', ('risk_measure', 'scenario') if multi_scen else 'risk_measure', 'dates' if has_dates else None
+    elif cls == 'PortfolioRiskResult':
+        if ori_cols is None:
+            raise ValueError('columns of dataframe required to get default pivots')
+        portfolio_names = list(filter(lambda x: 'portfolio_name_' in x, ori_cols))
+        port_and_inst_names = portfolio_names + ['instrument_name']
+        pivot_rules = [
+            # has_dates, multi_measures,  simple_port, multi_scen
+            # output: (value,index,columns)
+            [True, True, None, False, ('value', 'dates', port_and_inst_names + ['risk_measure'])],
+            [True, False, None, False, ('value', 'dates', port_and_inst_names)],
+            [False, False, False, False, ('value', portfolio_names, 'instrument_name')],
+            [False, None, None, False, ('value', port_and_inst_names, 'risk_measure')],
+
+            [True, True, None, True, ('value', 'dates', port_and_inst_names + ['risk_measure', 'scenario'])],
+            [True, False, None, True, ('value', 'dates', port_and_inst_names + ['scenario'])],
+            [False, True, None, True, ('value', port_and_inst_names, ['risk_measure', 'scenario'])],
+            [False, False, None, True, ('value', port_and_inst_names, 'scenario')],
+        ]
+
+        def match(rule_value, check_value) -> bool:
+            if rule_value is None:
+                return True
+            elif callable(rule_value):
+                return rule_value(check_value)
+            else:
+                return rule_value == check_value
+
+        for rule in pivot_rules:
+            [rule_has_dates, rule_multi_measures, rule_simple_port, rule_multi_scen, rule_output] = rule
+            if match(rule_has_dates, has_dates) and match(rule_multi_measures, multi_measures) and \
+                    match(rule_simple_port, simple_port) and match(rule_multi_scen, multi_scen):
+                return rule_output
+        return None, None, None
+
+
+def pivot_to_frame(df, values, index, columns, aggfunc):
+    try:
+        pivot_df = df.pivot_table(values=values, index=index, columns=columns, aggfunc=aggfunc)
+    except ValueError:
+        raise RuntimeError('Unable to successfully pivot data')
+
+    try:  # attempt to correct order of index
+        if index is not None:
+            idx = df.set_index(list(pivot_df.index.names)).index.unique()
+            pivot_df = pivot_df.reindex(index=idx)
+        if columns is not None:
+            cols = df.set_index(list(pivot_df.columns.names)).index.unique()
+            pivot_df = pivot_df.reindex(columns=cols)
+        return pivot_df
+    except KeyError:
+        return pivot_df
+
+
 def _compose(lhs: ResultInfo, rhs: ResultInfo) -> ResultInfo:
     if isinstance(lhs, ScalarWithInfo):
         if isinstance(rhs, ScalarWithInfo):
@@ -110,15 +168,15 @@ def _risk_keys_compatible(lhs, rhs) -> bool:
     return historical_risk_key(lhs.risk_key).ex_measure == historical_risk_key(rhs.risk_key).ex_measure
 
 
-def _value_for_risk_measure(res: dict, risk_measure: Union[Iterable, RiskMeasure]) -> dict:
+def _value_for_measure_or_scen(res: dict, item: Union[Iterable, RiskMeasure, Scenario]) -> dict:
     result = copy.copy(res)
-    if isinstance(risk_measure, Iterable):
+    if isinstance(item, Iterable):
         for value in list(result):
-            if value not in risk_measure:
+            if value not in item:
                 del result[value]
     else:
         for value in list(result):
-            if value != risk_measure:
+            if value != item:
                 del result[value]
     return result
 
@@ -128,8 +186,13 @@ class PricingFuture(Future):
 
     def __init__(self, result: Optional[Any] = __RESULT_SENTINEL):
         super().__init__()
+        self.__pricing_context = None
+
         if result is not self.__RESULT_SENTINEL:
             self.set_result(result)
+        else:
+            from gs_quant.markets import PricingContext
+            self.__pricing_context = weakref.ref(PricingContext.current.active_context)
 
     def __add__(self, other):
         if isinstance(other, (int, float)):
@@ -162,9 +225,10 @@ class PricingFuture(Future):
 
         Exception: If the call raised then that exception will be raised.
         """
-        from gs_quant.markets import PricingContext
-        if not self.done() and PricingContext.current.active_context.is_entered:
-            raise RuntimeError('Cannot evaluate results under the same pricing context being used to produce them')
+        if not self.done():
+            pricing_context = self.__pricing_context() if self.__pricing_context else None
+            if pricing_context is not None and pricing_context.is_entered:
+                raise RuntimeError('Cannot evaluate results under the same pricing context being used to produce them')
 
         return super().result(timeout=timeout)
 
@@ -213,6 +277,13 @@ class MultipleRiskMeasureResult(dict):
                                                                      for k, v in self.items()))
             else:
                 raise ValueError('Can only index by date on historical results')
+        elif is_instance_or_iterable(item, Scenario):
+            if all(isinstance(v, (MultipleScenarioResult)) for v in self.values()):
+                return MultipleRiskMeasureResult(self.__instrument, ((k, _value_for_measure_or_scen(v, item))
+                                                                     for k, v in self.items()))
+            else:
+                raise ValueError('Can only index by scenario on multiple scenario results')
+
         else:
             return super().__getitem__(item)
 
@@ -287,6 +358,13 @@ class MultipleRiskMeasureResult(dict):
 
         return tuple(sorted(dates))
 
+    @property
+    def _multi_scen_key(self) -> Iterable[Scenario]:
+        for value in self.values():
+            if isinstance(value, MultipleScenarioResult):
+                return tuple(value.scenarios)
+        return tuple()
+
     def to_frame(self, values='default', index='default', columns='default', aggfunc=pd.unique,
                  display_options: DisplayOptions = None):
         df = pd.DataFrame.from_records(self._to_records({}, display_options=display_options))
@@ -295,21 +373,14 @@ class MultipleRiskMeasureResult(dict):
         elif values == 'default' and index == 'default' and columns == 'default':
             if 'mkt_type' in df.columns:
                 return df.set_index('risk_measure')
-            values = 'value'
-            columns = 'risk_measure'
-            index = 'dates' if 'dates' in df.columns else None
+            values, columns, index = get_default_pivots('MultipleRiskMeasureResult', multi_measures=True,
+                                                        has_dates='dates' in df.columns,
+                                                        multi_scen='scenario' in df.columns)
         else:
             values = 'value' if values == 'default' or values is ['value'] else values
             index = None if index == 'default' else index
             columns = None if columns == 'default' else columns
-        pivot_df = df.pivot_table(values=values, index=index, columns=columns, aggfunc=aggfunc)
-        if index is not None:
-            idx = df.set_index(list(pivot_df.index.names)).index.unique()
-            pivot_df = pivot_df.reindex(index=idx)
-        if columns is not None:
-            cols = df.set_index(list(pivot_df.columns.names)).index.unique()
-            pivot_df = pivot_df.reindex(columns=cols)
-        return pivot_df
+        return pivot_to_frame(df, values, index, columns, aggfunc)
 
     def _to_records(self, extra_dict, display_options: DisplayOptions = None):
         return list(chain.from_iterable(
@@ -326,10 +397,7 @@ class MultipleRiskMeasureFuture(CompositeResultFuture):
 
     def __add__(self, other):
         result = self.result() + other.result() if isinstance(other, MultipleRiskMeasureFuture) else other
-        ret = MultipleRiskMeasureFuture(self.__instrument, {k: PricingFuture(v) for k, v in result.items()})
-        ret._set_result()
-
-        return ret
+        return MultipleRiskMeasureFuture(self.__instrument, {k: PricingFuture(v) for k, v in result.items()})
 
     def _set_result(self):
         self.set_result(MultipleRiskMeasureResult(self.__instrument,
@@ -341,6 +409,51 @@ class MultipleRiskMeasureFuture(CompositeResultFuture):
         return self.__measures_to_futures
 
 
+class MultipleScenarioResult(dict):
+    def __init__(self, instrument, dict_values: Iterable):
+        super().__init__(dict_values)
+        self.__scenarios = self.keys()
+        self.__instrument = instrument
+
+    def __getitem__(self, item):
+        if is_instance_or_iterable(item, dt.date):
+            if all(isinstance(v, (DataFrameWithInfo, SeriesWithInfo)) for v in self.values()):
+                return MultipleScenarioResult(self.__instrument,
+                                              ((k, _value_for_date(v, item)) for k, v in self.items()))
+            else:
+                raise ValueError('Can only index by date on historical results')
+        return super().__getitem__(item)
+
+    def to_frame(self, values='default', index='default', columns='default', aggfunc=pd.unique,
+                 display_options: DisplayOptions = None):
+        df = pd.DataFrame.from_records(self._to_records({}, display_options=display_options))
+        if values is None and index is None and columns is None:
+            return df
+        elif values == 'default' and index == 'default' and columns == 'default':
+            if 'mkt_type' in df.columns:
+                return df.set_index('scenario')
+            values, columns, index = get_default_pivots('MultipleScenarioResult', has_dates='dates' in df.columns,
+                                                        multi_measures=False, multi_scen=True)
+        else:
+            values = 'value' if values == 'default' or values is ['value'] else values
+            index = None if index == 'default' else index
+            columns = None if columns == 'default' else columns
+        return pivot_to_frame(df, values, index, columns, aggfunc)
+
+    @property
+    def instrument(self):
+        return self.__instrument
+
+    @property
+    def scenarios(self):
+        return self.__scenarios
+
+    def _to_records(self, extra_dict, display_options: DisplayOptions = None):
+        return list(chain.from_iterable(
+            [dict(item, **{'scenario': scen}) for item in self[scen]._to_records(extra_dict, display_options)]
+            for scen in self))
+
+
 class HistoricalPricingFuture(CompositeResultFuture):
 
     def _set_result(self):
@@ -351,9 +464,14 @@ class HistoricalPricingFuture(CompositeResultFuture):
             _logger.error(f'Historical pricing failed: {results[0]}')
             self.set_result(results[0])
         else:
-            result = MultipleRiskMeasureResult(base.instrument,
-                                               {k: base[k].compose(r[k] for r in results) for k in base.keys()}) \
-                if isinstance(base, MultipleRiskMeasureResult) else base.compose(results)
+            if isinstance(base, MultipleRiskMeasureResult):
+                result = MultipleRiskMeasureResult(base.instrument,
+                                                   {k: base[k].compose(r[k] for r in results) for k in base.keys()})
+            elif isinstance(base, MultipleScenarioResult):
+                result = MultipleScenarioResult(base.instrument,
+                                                {k: base[k].compose(r[k] for r in results) for k in base.keys()})
+            else:
+                result = base.compose(results)
             self.set_result(result)
 
 
@@ -435,15 +553,35 @@ class PortfolioRiskResult(CompositeResultFuture):
                     else:
                         futures.append(MultipleRiskMeasureFuture(priceable,
                                                                  {k: PricingFuture(v) for k, v in
-                                                                  _value_for_risk_measure(result, item).items()}))
+                                                                  _value_for_measure_or_scen(result, item).items()}))
 
                 risk_measure = tuple(item) if isinstance(item, Iterable) else (item,)
                 return PortfolioRiskResult(self.__portfolio, risk_measure, futures)
 
+        elif is_instance_or_iterable(item, Scenario):
+            '''Slicing a list of scenarios'''
+            if isinstance(item, Iterable) and any(it not in self._multi_scen_key for it in item):
+                raise ValueError('{} not computed'.format(item))
+            else:
+                if item not in self._multi_scen_key:
+                    raise ValueError('{} not computed'.format(item))
+
+            if len(self._multi_scen_key) == 0:  # field not used if not multiscenario
+                return self
+            else:
+                for idx, priceable in enumerate(self.portfolio):
+                    result = self.__result(PortfolioPath(idx))
+                    if isinstance(result, (MultipleRiskMeasureResult, PortfolioRiskResult)):
+                        futures.append(result[item])
+                    elif isinstance(result, MultipleScenarioResult):
+                        # scenario = tuple(item) if isinstance(item, Iterable) else (item,)
+                        futures.append(PricingFuture(_value_for_measure_or_scen(result, item)))
+
+                return PortfolioRiskResult(self.__portfolio, self.risk_measures, futures)
         elif is_instance_or_iterable(item, dt.date):
             for idx, _ in enumerate(self.portfolio):
                 result = self.__result(PortfolioPath(idx))
-                if isinstance(result, (MultipleRiskMeasureResult, PortfolioRiskResult)):
+                if isinstance(result, (MultipleRiskMeasureResult, PortfolioRiskResult, MultipleScenarioResult)):
                     futures.append(PricingFuture(result[item]))
                 elif isinstance(result, (DataFrameWithInfo, SeriesWithInfo)):
                     futures.append(PricingFuture(_value_for_date(result, item)))
@@ -582,6 +720,15 @@ class PortfolioRiskResult(CompositeResultFuture):
         except TypeError:
             return tuple()
 
+    @property
+    def _multi_scen_key(self) -> Iterable[Scenario]:
+        for result in self.__results():
+            if isinstance(result, MultipleScenarioResult):
+                return tuple(result.scenarios)
+            elif isinstance(result, (MultipleRiskMeasureResult, PortfolioRiskResult)):
+                return result._multi_scen_key
+        return tuple()
+
     def result(self, timeout: Optional[int] = None):
         super().result(timeout=timeout)
         return self
@@ -598,82 +745,33 @@ class PortfolioRiskResult(CompositeResultFuture):
         else:
             return aggregate_results(self.__results(), allow_mismatch_risk_keys=allow_mismatch_risk_keys)
 
+    def _to_records(self, display_options: DisplayOptions = None):
+        def get_records(rec):
+            temp = []
+            for f in rec.futures:
+                if isinstance(f.result(), ResultInfo) or isinstance(f.result(), MultipleRiskMeasureResult) \
+                        or isinstance(f.result(), MultipleScenarioResult):
+                    temp.append(f.result())
+                else:
+                    temp.extend(get_records(f.result()))
+            return temp
+
+        future_records = get_records(self)
+        portfolio_records = self.__portfolio._to_records()
+        records = []
+        if len(future_records) == len(portfolio_records):
+            for i in range(len(future_records)):
+                records.extend(future_records[i]._to_records({**portfolio_records[i]}, display_options))
+        return records
+
     def to_frame(self, values='default', index='default', columns='default', aggfunc=sum,
                  display_options: DisplayOptions = None):
-        def get_name(obj, idx):
-            if isinstance(obj, InstrumentBase) and hasattr(obj, 'type'):
-                type_name = obj.type.name if isinstance(obj.type, AssetType) else obj._type
-            else:
-                type_name = 'Portfolio'
-            return f'{type_name}_{idx}' if obj.name is None else obj.name
-
-        def get_df(priceable):
-            portfolio_paths = list(unique_everseen(p.path[:-1] for p in priceable.all_paths if len(p.path) > 1))
-            inst_paths = [p.path for p in priceable.all_paths if len(p.path) == 1]
-            final_records = []
-            if len(portfolio_paths):
-                for portfolio_path in portfolio_paths:
-                    curr_res = self.__result(PortfolioPath(portfolio_path))
-                    portfolio_info = {}
-                    for p_idx, p in enumerate(portfolio_path):
-                        portfolio = self.__result(PortfolioPath(portfolio_path[:p_idx + 1])).__portfolio
-                        portfolio_info[f'portfolio_name_{p_idx}'] = get_name(portfolio, p)
-
-                    records = list(chain.from_iterable((i._to_records(
-                        {**portfolio_info, 'instrument_name': get_name(curr_res.__portfolio[slice_idx], slice_idx)},
-                        display_options) for slice_idx, i in enumerate(curr_res))))
-
-                    records = list(filter(lambda x: x is not None, records))
-                    for rec_idx, rec in enumerate(records):
-                        if 'risk_measure' not in rec:
-                            records[rec_idx]['risk_measure'] = curr_res.risk_measures[0]
-
-                    final_records.extend(records)
-
-            if len(inst_paths):
-                for path in inst_paths:
-                    final_records.extend(self.__result(PortfolioPath(path))._to_records(
-                        {'instrument_name': get_name(priceable[path[0]], path[0])}, display_options))
-
-            final_records = list(filter(lambda x: x is not None, final_records))
-
-            if len(final_records) > 0:
-                final_df = pd.DataFrame.from_records(final_records)
-                if 'risk_measure' not in final_df.columns.values:
-                    final_df['risk_measure'] = self.risk_measures[0]
-                return final_df
-
-        def get_default_pivots(ori_cols, has_dates: bool, multi_measures: bool, simple_port: bool) -> tuple:
-            portfolio_names = list(filter(lambda x: 'portfolio_name_' in x, ori_cols))
-            port_and_inst_names = portfolio_names + ['instrument_name']
-            pivot_rules = [
-                # has_dates, multi_measures,  simple_port
-                # output: (value,index,columns)
-                [True, True, None, ('value', 'dates', port_and_inst_names + ['risk_measure'])],
-                [True, False, None, ('value', 'dates', port_and_inst_names)],
-                [False, False, False, ('value', portfolio_names, 'instrument_name')],
-                [False, True, False, ('value', port_and_inst_names, 'risk_measure')],
-                [False, None, True, ('value', 'instrument_name', 'risk_measure')],
-            ]
-
-            def match(rule_value, check_value) -> bool:
-                if rule_value is None:
-                    return True
-                elif callable(rule_value):
-                    return rule_value(check_value)
-                else:
-                    return rule_value == check_value
-
-            for rule in pivot_rules:
-                [rule_has_dates, rule_multi_measures, rule_simple_port, rule_output] = rule
-                if match(rule_has_dates, has_dates) and match(rule_multi_measures, multi_measures) and \
-                        match(rule_simple_port, simple_port):
-                    return rule_output
-            return None, None, None
-
-        ori_df = get_df(self.portfolio)
-
-        if ori_df is None:
+        final_records = self._to_records(display_options=display_options)
+        if len(final_records) > 0:
+            ori_df = pd.DataFrame.from_records(final_records)
+            if 'risk_measure' not in ori_df.columns.values:
+                ori_df['risk_measure'] = self.risk_measures[0]
+        else:
             return
         df_cols = list(ori_df.columns.values)
         # fill n/a values for different sub-portfolio depths
@@ -692,6 +790,7 @@ class PortfolioRiskResult(CompositeResultFuture):
         if values is None and index is None and columns is None:  # to_frame(None, None, None)
             return ori_df
         elif values == 'default' and index == 'default' and columns == 'default':  # to_frame()
+            multi_scen = len(self._multi_scen_key) > 1
             has_bucketed = True if 'mkt_type' in df_cols else False
             has_cashflows = True if 'payment_amount' in df_cols else False
             multi_rm = True if len(self.risk_measures) > 1 else False
@@ -699,20 +798,13 @@ class PortfolioRiskResult(CompositeResultFuture):
             if has_bucketed or has_cashflows:
                 return ori_df.set_index(other_cols)
             else:
-                values, index, columns = get_default_pivots(df_cols, has_dt, multi_rm, port_depth_one)
+                values, index, columns = get_default_pivots('PortfolioRiskResult', has_dates=has_dt,
+                                                            multi_measures=multi_rm, simple_port=port_depth_one,
+                                                            multi_scen=multi_scen, ori_cols=df_cols)
         else:  # user defined pivoting
             values = 'value' if values == 'default' or values is ['value'] else values
 
-        try:
-            pivot_df = ori_df.pivot_table(values=values, index=index, columns=columns, aggfunc=aggfunc)
-        except ValueError:
-            raise RuntimeError('Unable to successfully pivot data')
-        try:  # attempt to correct order of index
-            ori_index = ori_df.set_index(list(pivot_df.index.names)).index.unique()
-            ori_columns = ori_df.set_index(list(pivot_df.columns.names)).index.unique()
-            return pivot_df.reindex(index=ori_index, columns=ori_columns)
-        except KeyError:
-            return pivot_df
+        return pivot_to_frame(ori_df, values, index, columns, aggfunc)
 
     def __paths(self, items: Union[int, slice, str, Priceable]) -> Tuple[PortfolioPath, ...]:
         if isinstance(items, int):
