@@ -14,7 +14,6 @@ specific language governing permissions and limitations
 under the License.
 """
 
-
 from abc import abstractmethod
 import backoff
 import certifi
@@ -76,7 +75,7 @@ class GsSession(ContextBase):
             )
 
     def __init__(self, domain: str, api_version: str = API_VERSION, application: str = DEFAULT_APPLICATION, verify=True,
-                 http_adapter: requests.adapters.HTTPAdapter = None, application_version=APP_VERSION):
+                 http_adapter: requests.adapters.HTTPAdapter = None, application_version=APP_VERSION, proxies=None):
         super().__init__()
         self._session = None
         self.domain = domain
@@ -85,6 +84,7 @@ class GsSession(ContextBase):
         self.verify = verify
         self.http_adapter = requests.adapters.HTTPAdapter(pool_maxsize=100) if http_adapter is None else http_adapter
         self.application_version = application_version
+        self.proxies = proxies
 
     @backoff.on_exception(lambda: backoff.expo(factor=2),
                           (requests.exceptions.HTTPError, requests.exceptions.Timeout),
@@ -110,6 +110,8 @@ class GsSession(ContextBase):
             self._session = requests.Session()
             if self.http_adapter is not None:
                 self._session.mount('https://', self.http_adapter)
+            if self.proxies is not None:
+                self._session.proxies = self.proxies
             self._session.verify = self.verify
             self._session.headers.update({'X-Application': self.application})
             self._session.headers.update({'X-Version': self.application_version})
@@ -437,14 +439,26 @@ try:
     class PassThroughGSSSOSession(KerberosSessionMixin, GsSession):
 
         def __init__(self, environment: str, token, api_version=API_VERSION,
-                     application=DEFAULT_APPLICATION, http_adapter=None):
+                     application=DEFAULT_APPLICATION, http_adapter=None, csrf_token=None):
             domain, verify = self.domain_and_verify(environment)
             GsSession.__init__(self, domain, api_version=api_version, application=application, verify=verify,
                                http_adapter=http_adapter)
 
             self.token = token
+            self.csrf_token = csrf_token
 
         def _authenticate(self):
-            self._handle_cookies(self.token)
+            if not (self.token and self.csrf_token):
+                self._handle_cookies(self.token)
+                return
+
+            cookie = requests.cookies.create_cookie(domain='.gs.com', name='GSSSO', value=self.token)
+            self._session.cookies.set_cookie(cookie)
+            if self.csrf_token:
+                cookie = requests.cookies.create_cookie(domain='.gs.com', name='MARQUEE-CSRF-TOKEN',
+                                                        value=self.csrf_token)
+                self._session.cookies.set_cookie(cookie)
+                self._session.headers.update({'X-MARQUEE-CSRF-TOKEN': self.csrf_token})
+
 except ModuleNotFoundError:
     pass
