@@ -16,13 +16,17 @@ under the License.
 
 import hashlib
 import json
+import os
 import pathlib
+from typing import List
 from unittest import mock
 
+import pytest
 from gs_quant.api.gs.risk import GsRiskApi
 from gs_quant.json_encoder import JSONEncoder
 from gs_quant.markets import PricingContext
 from gs_quant.session import Environment, GsSession
+from gs_quant.target.common import CompositeScenario
 
 
 def _remove_unwanted(json_text):
@@ -87,6 +91,9 @@ def get_risk_request_id(requests):
     """
     identifier = str(len(requests))
     for request in requests:
+        for pos in request.positions:
+            if pos.instrument.name is None:
+                raise ValueError('Positions must have names to be mocked')
         identifier += '_'
         identifier += '-'.join([pos.instrument.name for pos in request.positions])
         identifier += '-'.join([r.__repr__() for r in request.measures])
@@ -94,21 +101,33 @@ def get_risk_request_id(requests):
         today = PricingContext().pricing_date.strftime('%Y%b%d')
         identifier += 'today' if date == today else date
         if request.scenario is not None:
-            scenario_identifier = []
-            for k, v in request.scenario.scenario.as_dict().items():
-                if k != 'shocks':
-                    scenario_identifier.append(str(k) + "=" + str(v))
-                else:
-                    shock_value = 'shock_value' + "=" + str(v[0].shock.value)
-                    pattern = v[0].pattern
-                    shock_pattern = 'shock_pattern' + "=" + '-'.join(
-                        [str(m) for m in [pattern.mkt_type, pattern.mkt_asset, pattern.mkt_class]])
-                    scenario_identifier.append(shock_value + "+" + shock_pattern)
-            identifier += '+'.join(sorted(scenario_identifier))
+            if isinstance(request.scenario.scenario, CompositeScenario):
+                underlying_scenarios = request.scenario.scenario.scenarios
+                if any([scen.name is None for scen in underlying_scenarios]):
+                    raise RuntimeError('Please provide unique names for your scenarios for testing')
+                identifier += '+'.join(sorted([r.name for r in underlying_scenarios]))
+            else:
+                if request.scenario.scenario.name is None:
+                    raise RuntimeError('Please provide unique names for your scenarios for testing')
+                identifier += request.scenario.scenario.name
     return hashlib.md5(identifier.encode('utf-8')).hexdigest()
 
 
+@pytest.mark.last
+def test_all_cache_files_used():
+    # Important that this test runs last, it asserts all the test files are used so we can cleanup unused ones
+    saved_files = MockCalc.get_saved_files()
+    assert [] == saved_files, 'Did you accidentally commit with save_files=True?!'
+
+    unused_files = MockCalc.get_unused_files()
+    print(unused_files)
+    assert unused_files == [], 'Cleanup your unused test files!'
+
+
 class MockCalc:
+    __looked_at_files = {}
+    __saved_files = set()
+
     def __init__(self, mocker, save_files=False, paths=pathlib.Path(__file__).parents[1], application='gs-quant'):
         # do not save tests with save_files = True
         self.save_files = save_files
@@ -128,8 +147,10 @@ class MockCalc:
 
     def mock_calc(self, *args, **kwargs):
         request = kwargs.get('request') or args[0]
-        with open(self.paths / f'calc_cache/request{get_risk_request_id(request)}.json') \
-                as json_data:
+        request_id = get_risk_request_id(request)
+        file_name = f'request{request_id}.json'
+        with open(self.paths / f'calc_cache/{file_name}') as json_data:
+            MockCalc.__looked_at_files.setdefault(self.paths, set()).add(file_name)
             return json.load(json_data)
 
     def mock_calc_create_files(self, *args, **kwargs):
@@ -137,16 +158,35 @@ class MockCalc:
         # and switch to mock_calc
         def get_json(*i_args, **i_kwargs):
             this_json = gs_risk_api_exec(*i_args, **i_kwargs)
+            # Post process the json a bit to remove timing info that makes spurious diffs
+            for d in [d for a in this_json for b in a for c in b for d in c]:
+                for key, val in list(d.items()):
+                    if val is None or key in ['queueingTime', 'calculationTime']:
+                        del d[key]
             return this_json
 
         result_json = get_json(*args, **kwargs)
         request = kwargs.get('request') or args[0]
-
-        with open(self.paths / f'calc_cache/request{get_risk_request_id(request)}.json',
+        request_id = get_risk_request_id(request)
+        with open(self.paths / f'calc_cache/request{request_id}.json',
                   'w') as json_data:
+            MockCalc.__saved_files.add(request_id)
             json.dump(result_json, json_data)
 
         return result_json
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         pass
+
+    @staticmethod
+    def get_unused_files() -> List[str]:
+        unused_files = []
+        for test_path, files_used in MockCalc.__looked_at_files.items():
+            for test_file in os.listdir(test_path / 'calc_cache'):
+                if test_file.endswith('.json') and test_file not in files_used:
+                    unused_files.append(test_file)
+        return unused_files
+
+    @staticmethod
+    def get_saved_files() -> List[str]:
+        return list(MockCalc.__saved_files)
